@@ -21,3 +21,8 @@ NMSLIB <- NULL; SCP <- NULL;
     }
   }, silent=TRUE)
 }
+
+
+.onAttach <- function(libname, pkgname) {
+  packageStartupMessage("If the 'nmslibR' package gives the following error: 'attempt to apply non-function' then make sure to open a new R session and run 'reticulate::py_config()' before loading the package!")
+}
